/*
 * Created on 26-nov-2004
 *
 * Code of the acquire agent
 */
package agents;


import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.StringTokenizer;
import javax.comm.CommPortIdentifier;
import javax.comm.SerialPort;
import javax.comm.SerialPortEvent;
import javax.comm.SerialPortEventListener;
import javax.comm.UnsupportedCommOperationException;
import inputs.WeatherStations;
import alice.tucson.api.*;

/**
 * @author Pazzaglia Marco
 * @version 1.0
 */

public class AcquireAgent extends Agent {
    
    private static TupleCentreId Tc;
    private static CommPortIdentifier portID; 
    private static InputStream inputStream;
    private static OutputStream out;
    private static SerialPort PortaSeriale;
    private static int count=0;
    private static byte[] binReadBuffer= new byte[50];
    private static String strReadBuffer="";
    private static TupleCentreId tid;
    private static String WeatherStationModel;
    private static String Port;
    
    /**
     * Default constructor of the class
     * @param arg0 AgentId - the id of the Agent
     * @param tc the tuple center used
     * @param weatherStationModel the model of the weather station used or other input data methods
     * @param port the communication port used
     * @throws TucsonException
     */
    public AcquireAgent(AgentId arg0, TupleCentreId tc, String weatherStationModel, String port) throws TucsonException {
        super(arg0);
        WeatherStationModel = weatherStationModel;
        Tc = tc;
        Port = port;
    }

    public void body(){ 
        try {
            TucsonContext cn1 = Tucson.enterDefaultContext();
	        TupleCentreId tid1 = Tc;
	        portID = CommPortIdentifier.getPortIdentifier(Port);
            PortaSeriale = (SerialPort) portID.open("Data Acquisition Agent from "+Port+" with "+WeatherStationModel,2000);
            ReadCom(WeatherStationModel);
            
        } catch(Exception ex){
            System.out.println("Error AcquireAgent :"+ex);
        }
    }

    /**
     * Setting the port parameters and add the comm listeners
     * @param WeatherStationModel
     */
    public static void ReadCom(String WeatherStationModel) {
        try {
            inputStream = PortaSeriale.getInputStream();
            out = PortaSeriale.getOutputStream(); 
            PortaSeriale.addEventListener(new commListener());
        }
        catch (Exception e) {
            System.out.println("Errore ReadCom: "+e);
            }
        
        PortaSeriale.notifyOnDataAvailable(true);
        
        try {
            PortaSeriale.setSerialPortParams(9600,
            SerialPort.DATABITS_8,
            SerialPort.STOPBITS_1 ,
            SerialPort.PARITY_NONE);
            }
        
        catch (UnsupportedCommOperationException e) {
            }
    }
    
    
    /**
     * @author Administrator
     *
     * Inner class to specofy a comm listener for the serial port
     * This inner class sets an inputstream and give it to the parse method
     */
    public static class commListener implements SerialPortEventListener{ 
        public void serialEvent(SerialPortEvent event) {
            if(event.getEventType()==SerialPortEvent.DATA_AVAILABLE){
                try
                {   
                    int numBytes = 0;
                    while (inputStream.available() > 0)
                    {
                        strReadBuffer="";
                        numBytes = inputStream.read(binReadBuffer);
                        for (int i=0; i<20; i++)
                        {
                            strReadBuffer+=byteToStr(binReadBuffer[i])+" ";
                        }
 
                    }
                    count++;
                   try {
                    parseStream(strReadBuffer, WeatherStationModel, Tc);
                    }
                    catch (Exception e){System.out.println("Errore del parseStream: "+e);}
                }
                catch (IOException e){System.out.println("Errore ComListener");}
            }
        }
    } 
    
    
    /**
     * This procedure transform an hex byte to a decimal-format-string
     * @param b the hex-byte to convert
     * @return the decimal-format-string rappresentation of the input hex byte
     */
    static public String byteToStr(byte b) {
        // Returns hex String representation of byte b
        char hexDigit[] = {
                '0', '1', '2', '3', '4', '5', '6', '7',
                '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
        };
        char[] cArray = { hexDigit[(b >> 4) & 0x0f], hexDigit[b & 0x0f] };
       // output.append("\nbyteToStr: "+cArray[0]+cArray[1]+"\n");
        return ""+cArray[0]+cArray[1];
    }
    
    /**
     * This void make a first parse of the data and call the methods on the inputs class
     * relative of the right weatherstation model
     * @param strReadBuffer the string containing the input buffer data
     * @param WeatherStationModel the weather station model
     * @param tc the tuplecenter used
     */
    public static void parseStream(String strReadBuffer, String WeatherStationModel, TupleCentreId tc){
        
        StringTokenizer buff = new StringTokenizer(strReadBuffer);
        
        //choose the Weatherstation Type
        if (WeatherStationModel.compareTo("wmr918")==0){
		            String packetBody="";
		            String curToken = buff.nextToken();
		            if (curToken.compareTo("ff")!=0){
		            }
		            else{
		                curToken = buff.nextToken();
		                curToken = buff.nextToken();
		                int packetType = takePacketTypeNumber(curToken);
		                while(buff.hasMoreTokens()){
		                    packetBody += buff.nextToken()+" ";
		                }
		                try{
		                WeatherStations.oregonScientificWmr918(packetBody, packetType, tc);
		                } catch (Exception e){System.out.println("\nParsing errato: "+e);}
		            }
        }
        else {
            System.out.println("\nThis Weather Station is not actually supported");
        }
  
    }


    
    
    /**
     * @param curToken the current token to analyze
     * @return an int rapresenting the packet type
     */
    static public int takePacketTypeNumber(String curToken){  
        int out = Integer.parseInt(curToken.substring(1,2),16);
        return out;
    }
}
