/*
 * Created on 18-dic-2004
 *
 * Code of the Forecasts agent
 */
package agents;


import alice.tucson.api.*;
import alice.logictuple.*;

import java.lang.Float;

public class ForecastsAgent extends Agent {

    /**
     * @author Pazzaglia Marco
     * @version 1.0
     */
    private TupleCentreId Tid;
    private float AvgWind=0, AvgRain=0, AvgBaro=0;
    private float VarWind=0, VarRain=0, VarBaro=0;
    
    private float thresholdWind=8;//8 m/s = circa 30 Km/h
    private float thresholdRain=3;//3 mm/hr
    private float thresholdBaro=3;//3 MilliBar/ora
    
    private float thresholdVarWind=14;//14 m/s = circa 50 Km/h
    private float thresholdVarRain=1;//1 mm/hr
    private float thresholdVarBaro=2;//2 MilliBar/ora
                
    /**
     * Default constructor of the class
     * @param arg0 AgentId - the id of the Agent
     * @param gui the gui used
     * @param tid the tuple center used
     * @throws TucsonException
     */
    public ForecastsAgent(AgentId arg0, TupleCentreId tid) throws TucsonException {
        super(arg0);
        Tid = tid;  
    }

    public void body(){ 
        while (true){
            try { 
                
		        startAnalisis();
		        
		        TucsonContext context = Tucson.enterDefaultContext();
		        if (AvgWind>thresholdWind){
		            context.out(Tid, new LogicTuple("windAlarm",new Value(AvgWind)));
		            if (VarWind>thresholdVarWind){
			            context.out(Tid, new LogicTuple("windRafficaAlarm",new Value(VarWind)));
			        }
		        }
		        if (AvgRain>thresholdRain){
		            context.out(Tid, new LogicTuple("rainAlarm",new Value(AvgRain)));
		            if (VarRain>thresholdVarRain){
			            context.out(Tid, new LogicTuple("rainAlluvioneAlarm",new Value(AvgWind)));
			        }
		        }
		        if (AvgBaro>thresholdBaro){
		            context.out(Tid, new LogicTuple("baroAlarm",new Value(AvgBaro)));
		            if (VarBaro>thresholdVarBaro){
			            context.out(Tid, new LogicTuple("baroSbalzoAlarm",new Value(AvgWind)));
			        }
		        }
		    } catch (Exception ex){System.out.println(ex);}
        }  
    }
    
    /**
     * This void start the analisis of the Forecasts agent
     * and interact with the tuple center forecasts
     */
    public void startAnalisis(){
        String[] fields = new String[10];
	    int timesToReadWind;
	    int var = 5;
	    float[] currValue = new float[var];
	    int timesToReadWindMedia;
	    
	    try { 
	        TucsonContext context = Tucson.enterDefaultContext();
	        
	            LogicTuple WindTrend = inp(Tid, new LogicTuple("windTrend",
	                    new Var("MediaVento"),
	                    new Var("VarianzaVento")));

	            LogicTuple BaroTrend = inp(Tid, new LogicTuple("baroTrend",
	                    new Var("MediaBaro"),
	                    new Var("VarianzaBaro")));
	            
	            LogicTuple RainTrend = inp(Tid, new LogicTuple("rainTrend",
	                    new Var("MediaPioggia"),
	                    new Var("VarianzaPioggia")));
	            
	            if (WindTrend==null){
	                AvgWind = 0;
	                VarWind = 0;
	                } else {
	                AvgWind =(new Float(WindTrend.getArg(0).toString())).floatValue();
	                VarWind =(new Float(WindTrend.getArg(1).toString())).floatValue();
	                }
	            if (BaroTrend==null){
	                AvgBaro = 0;
	                VarWind = 0;
	                } else {
	                AvgBaro =(new Float(BaroTrend.getArg(0).toString())).floatValue();
	                VarWind =(new Float(BaroTrend.getArg(1).toString())).floatValue();
	                }
	            if (RainTrend==null){
	                AvgRain = 0;
	                VarRain = 0;
	                } else {
	                AvgRain =(new Float(RainTrend.getArg(0).toString())).floatValue();
	                VarRain =(new Float(RainTrend.getArg(1).toString())).floatValue();
	                }
	    } catch (Exception ex){System.out.println(ex);}

    }
    
    
   
    
}