/*
 * Created on 26-nov-2004
 *
 * Code of the Initializer agent
 */
package agents;

import gui.GuiSwing;
import alice.tucson.api.*;


public class Initializer extends Agent {
    
    /**
     * @author Pazzaglia Marco
     * @version 1.0
     */
    
    private static TupleCentreId[] Tc = new TupleCentreId[10];
    private static String[] tCtoStart;
    private static GuiSwing myGui = new GuiSwing();
    
    /**
     * Default constructor of the class
     * @param arg0 AgentId - the id of the Agent
     * @param TCToStart an array containing the name of the tuple center/s to start
     * @param gui the gui used
     * @throws TucsonException
     */
    public Initializer(AgentId arg0, String[] TCtoStart, GuiSwing gui) throws TucsonException {
        super(arg0);
        myGui = gui;
        tCtoStart = TCtoStart;
    }
    
    public void body(){ 
        
        try {
            TucsonContext cn1 = Tucson.enterDefaultContext();
            
            //facciamo partire tutti i tuple center passati
            for (int tcCount=0; tcCount<=tCtoStart.length-1; tcCount++){
                Tc[tcCount] = new TupleCentreId(tCtoStart[tcCount]);
            }
            
            //Settaggio delle azioni del Tuple center con ReSpecT
            setSpec(Tc[0], "reaction(out(T),( out_tc(forecasts, T))).");
            setSpec(Tc[2], "reaction(out(T),( out_tc(view, T))).");
           
            
            //inizializzazione e lancio degli agenti
            /////////////////////////////////////////
            
            //agente per l'acquisizione
            AgentId AcquireAgent=new AgentId("acquireAgent");
            new AcquireAgent(AcquireAgent, Tc[0], "wmr918", "COM1").spawn();
            
            //agenti di visualizzazione
            AgentId GuiWindAgent=new AgentId("guiWindAgent");
            new ViewWindAgent(GuiWindAgent, myGui, Tc[1]).spawn();
            
            AgentId GuiStatusAgent=new AgentId("guiStatusAgent");
            new ViewStatusAgent(GuiStatusAgent, myGui, Tc[1]).spawn();
            
            AgentId GuiRainAgent=new AgentId("guiRainAgent");
            new ViewRainAgent(GuiRainAgent, myGui, Tc[1]).spawn();
            
            AgentId GuiOutAgent=new AgentId("guiOutAgent");
            new ViewOutAgent(GuiOutAgent, myGui, Tc[1]).spawn();
            
            AgentId GuiInAgent=new AgentId("guiInAgent");
            new ViewInAgent(GuiInAgent, myGui, Tc[1]).spawn();
            
            //AgentId GuiExtraAgent=new AgentId("guiExtraAgent");
            //new ViewExtraAgent(GuiExtraAgent, myGui).spawn();
            
            AgentId GuiBattAgent=new AgentId("guiBattAgent");
            new ViewBattAgent(GuiBattAgent, myGui, Tc[1]).spawn();
            
            //agenti di previsione
            AgentId WindAnalyzAgent=new AgentId("windAnalyzerAgent");
            new WindAnalyzerAgent(WindAnalyzAgent, Tc[2]).spawn();
            
            AgentId RainAnalyzAgent=new AgentId("rainAnalyzerAgent");
            new RainAnalyzerAgent(WindAnalyzAgent, Tc[2]).spawn();
            
            AgentId BaroAnalyzAgent=new AgentId("windAnalyzerAgent");
            new BaroAnalyzerAgent(WindAnalyzAgent, Tc[2]).spawn();
            
            AgentId ForeAgent=new AgentId("forecastsAgent");
            new ForecastsAgent(WindAnalyzAgent, Tc[2]).spawn();
            
            AgentId AlarmsAgent=new AgentId("alarmAgent");
            new AlarmAgent(AlarmsAgent, myGui, Tc[2]).spawn();
            
            
        } catch(Exception ex){
            System.out.println("Error Initializer :"+ex);
        }
        
        
        
    }
    
}
