/*
 * Created on 18-dic-2004
 *
 * Code of the inputs weather station models
 */
package inputs;


/**
 * @author Pazzaglia Marco
 * @version 1.0
 */

import java.util.*;
import java.text.*;
import alice.tucson.api.*;
import alice.logictuple.*;

public class WeatherStations {

    /**
     * This void parse the packets of the Oregon Scientific WMR 918 Weather Station
     * @param packetBody the string containing the body of the packet
     * @param packetType the packet type
     * @param tc the tuple center used t put the logic tuple after the parsing
     */
    public static void oregonScientificWmr918(String packetBody, int packetType, TupleCentreId tc){
        
        int BatteryStatus=0;
        int GustDirectionBC=0, GustDirectionA=0, GustSpeedAB=0, SignWindChill=0, WindChill=0;
        int RainRateBC=0, RainRateA=0, RainRate=0, BucketTips=0, RainTotalCD=0, RainTotalAB=0, RainTotal=0, RainTotalYestCD=0, RainTotalYestAB=0, RainTotalYest=0;
        int TotalResetMin=0, TotalResetHour=0, TotalResetDay=0, TotalResetMonth=0, TotalResetYear=0;
        int SignInTemp=0, SignOutTemp=0, RelHum=0, DewPoint=0, Tendency=0;
        int BaroPressBC=0, BaroPressA=0, BaroPress=0;
        int SensorNumber=0, MinuteChime=0, Hour=0, Day=0, Month=0, Year=0;
        
        float AverageSpeedBC=0, AverageSpeedA=0, GustDirection=0, GustSpeed=0, GustSpeedC=0, AverageSpeed=0;;
        float OutTemp=0, OutTempBC=0, OutTempA=0;
        float InTemp=0, InTempBC=0, InTempA=0, SeaLevel=0, SeaLevelCD=0, SeaLevelAB=0, SeaLevelEF=0;
        
        
        String GustDirAndSpeed="", AverageSpeedAndSign="", Data="";
        String RainRateAndBucket="";
        String BatteryAndSensorNum="", OutTempAndSign="";
        String InTempAndSign="", BaroPressAndTendency="", BaroPressBinA="";
        
        Date now = new Date();
        SimpleDateFormat dateformat = new SimpleDateFormat ("dd MMMM yyyy - HH:mm.ss");
        Data = dateformat.format(now);
        
        StringTokenizer packetData = new StringTokenizer(packetBody);
        
        switch(packetType){
		case 0:		
				    /*    
			         Type 00
			         
			         Example: 00 90 01 00 00 00 07 96
			         
			         Byte	Nibble	Bit	Meaning
			         00. 1	Bx		Battery status. Higher value == lower battery volt
			         00. 1   xB		Unknown
			         00. 2	DD		Gust direction, bc of 0<abc<359 degrees
			         00. 3	xD		Gust direction, a  of 0<abc<359 degrees
			         00. 3	Dx		Gust speed, c  of 0<ab.c<56 m/s
			         00. 4	DD		Gust speed, ab of 0<ab.c<56 m/s
			         00. 5	DD		Average speed, bc  of 0<ab.c<56 m/s
			         00. 6	xD		Average speed, a of 0<ab.c<56 m/s
			         00. 6	Bx	3	Sign of wind chill, 1 = negative
			         00. 7	DD		Wind chill
			         
			         Example: gust at 190 degrees, 0 m/s, average 0, wind chill 7 Celsius
			         */
				{
		    try {
				
		    		if (packetData.hasMoreTokens()){
			            BatteryStatus = Integer.parseInt(packetData.nextToken());
			        }
			        if (packetData.hasMoreTokens()){
			            GustDirectionBC = Integer.parseInt(packetData.nextToken());
			            GustDirAndSpeed = packetData.nextToken();
			            GustDirectionA = Integer.parseInt(GustDirAndSpeed.substring(1,2));
			            GustDirection = (GustDirectionA*100)+GustDirectionBC;
			        }
			        if (packetData.hasMoreTokens()){
			            GustSpeedC = Integer.parseInt(GustDirAndSpeed.substring(0,1));
			            GustSpeedAB = Integer.parseInt(packetData.nextToken());
			            GustSpeed = (GustSpeedAB)+(GustSpeedC/10);
			        }
			        if (packetData.hasMoreTokens()){
			            AverageSpeedBC = Integer.parseInt(packetData.nextToken());
			        }
			        
			        if (packetData.hasMoreTokens()){
			            AverageSpeedAndSign = packetData.nextToken();
			            AverageSpeedA = Integer.parseInt(AverageSpeedAndSign.substring(1,2));
			            AverageSpeed = (AverageSpeedA*10)+(AverageSpeedBC/10);
			        }
			        if (packetData.hasMoreTokens()){
			            SignWindChill = Integer.parseInt(AverageSpeedAndSign.substring(0,1));
			        }
			        
			        if (packetData.hasMoreTokens()){
			            WindChill = Integer.parseInt(packetData.nextToken());
			        }
			        
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("vento",new Value(Data), new Value(GustDirection), new Value(GustSpeed), new Value(AverageSpeed), new Value(WindChill)));
//			            context.out(tc, new LogicTuple("ventoFreddo",new Value(Data), new Value(WindChill)));
			        
			            }
			            catch (Exception e){System.out.println("Errore parse 0: "+e);}
				}
        			
        break;
        case 1:		
		            /*    
		            Type 01
		            
		            Example: 00 92 62 02 00 00 00 15 21 09 03 00 37
		            		  00 00 90 42 02 00 10 a4 	
		            Byte	Nibble	Bit	Meaning
		            01. 1	Bx		Battery status. Higher value == lower battery volt
		            01. 1   xB		Unknown
		            01. 2	DD		Rain rate, bc of 0<abc<999 mm/hr
		            01. 3	xD		Rain rate, a  of 0<abc<999 mm/hr
		            01. 3	Dx		Bucket tips since ??
		            01. 4	DD		Rain Total, cd of 0<abcd<9999 mm (?)
		            01. 5	DD		Rain Total, ab of 0<abcd<9999 mm (?)
		            01. 6	DD		Rain Yesterday, cd of 0<abdc<9999 mm (?)
		            01. 7	DD		Rain Yesterday, ab of 0<abcd<9999 mm (?)
		            01. 8	DD		Total reset minute
		            01. 9	DD		Total reset hour
		            01.10	DD		Total reset day
		            01.11	DD		Total reset month
		            01.12	DD		Total reset year 
		            
		            Example: 292 mm/hr, 0 yesterday, 2mm total since 21:15 9-Mar-2000
		            */
        		{try {
		            if (packetData.hasMoreTokens()){
		                BatteryStatus = Integer.parseInt(packetData.nextToken().substring(0,1));
		            }
		            if (packetData.hasMoreTokens()){
		                RainRateBC = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                RainRateAndBucket = packetData.nextToken();
		                RainRateA = Integer.parseInt(RainRateAndBucket.substring(1,2));
		                RainRate = (RainRateA*100)+RainRateBC;
		            }
		            if (packetData.hasMoreTokens()){
		                BucketTips = Integer.parseInt(RainRateAndBucket.substring(0,1));
		            }
		            if (packetData.hasMoreTokens()){
		                RainTotalCD = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                RainTotalAB = Integer.parseInt(packetData.nextToken());
		                RainTotal = (RainTotalAB*100)+(RainTotalCD);
		            }
		            if (packetData.hasMoreTokens()){
		                RainTotalYestCD = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                RainTotalYestAB = Integer.parseInt(packetData.nextToken());
		                RainTotalYest = (RainTotalYestAB*100)+(RainTotalYestCD);
		            }
		            /*if (packetData.hasMoreTokens()){
		                TotalResetMin = Integer.parseInt(packetData.nextToken());
		            }    
		            if (packetData.hasMoreTokens()){
		                TotalResetHour = Integer.parseInt(packetData.nextToken(),16);
		            }
		            if (packetData.hasMoreTokens()){
		                TotalResetDay = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                TotalResetMonth = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                TotalResetYear = Integer.parseInt(packetData.nextToken());
		            }
		            */
		            
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("pioggia",new Value(Data), new Value(RainRate), new Value(RainTotal), new Value(RainTotalYest)));
			            }
			            catch (Exception e){System.out.println("Errore parse 1: "+e);}
			            
        		}
        			
        break;
        case 2:		
		            /*    
		            Example: 81 71 00 87 05 ff
		            
		            Byte	Nibble	Bit	Meaning
		            02. 1	Bx		Battery status. Higher value == lower battery volt
		            02. 1   xB		Unknown
		            02. 1	xB		Sensor number bit encoded, 4=3 2=2 1=1
		            02. 2	DD		Outside temp, bc of -?<ab.c<? Celsius
		            02. 3	xD		Outside temp, a  of -?<ab.c<? Celsius
		            02. 3	Bx	3	Sign of outside temp, 1 = negative
		            02. 4	DD		Relative humidity, ab of ?<ab<? percent
		            02. 5	DD		Dew point, ab of 0<ab<? Celsius
		            
		            Example: 7.1 Celsius, 87% humidity, dew point 5 Celsius
		            */
            
        		/*{try {
		            if (packetData.hasMoreTokens()){
		                BatteryAndSensorNum = packetData.nextToken();
		                BatteryStatus = Integer.parseInt(BatteryAndSensorNum.substring(0,1));
			            SensorNumber = Integer.parseInt(BatteryAndSensorNum.substring(1,2));
		            }
		            
		            
		            if (packetData.hasMoreTokens()){
		                OutTempBC = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                OutTempAndSign = packetData.nextToken();
			            OutTempA = Integer.parseInt(BatteryAndSensorNum.substring(1,2));
			            SignOutTemp = Integer.parseInt(BatteryAndSensorNum.substring(0,1));
			            OutTemp = (SignOutTemp==1) ? ((-1)*(OutTempBC/10+OutTempA*10)) : (OutTempBC/10+OutTempA*10);
		            }
		            

		            if (packetData.hasMoreTokens()){
		                RelHum = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                DewPoint = Integer.parseInt(packetData.nextToken());
		            }
        			
		            
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("extraSensor",new Value(Data), new Value(OutTemp), new Value(RelHum), new Value(DewPoint)));
			            }
			            catch (Exception e){System.out.println("Errore parse 2: "+e);}
			            
        		}*/
        			
        break;
        case 3:	
		                
		           /* Example: 01 71 00 87 05 ff
		            
		            Byte	Nibble	Bit	Meaning
		            03. 1	Bx		Battery status. Higher value == lower battery volt
		            03. 1   xB		Unknown
		            03. 2	DD		Outside temp, bc of -?<ab.c<? Celsius
		            03. 3	xD		Outside temp, a  of -?<ab.c<? Celsius
		            03. 3	Bx	3	Sign of outside temp, 1 = negative
		            03. 4	DD		Relative humidity, ab of ?<ab<? percent
		            03. 5	DD		Dew point, ab of 0<ab<? Celsius
		            
		            Example: 7.1 Celsius, 87% humidity, dew point 5 Celsius
		            */
            
        		{
		            if (packetData.hasMoreTokens()){
		                BatteryAndSensorNum = packetData.nextToken();
			            BatteryStatus = Integer.parseInt(BatteryAndSensorNum.substring(0,1));
		            }
		            
		            if (packetData.hasMoreTokens()){
		                OutTempBC = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                OutTempAndSign = packetData.nextToken();
			            OutTempA = Integer.parseInt(OutTempAndSign.substring(1,2));
			            SignOutTemp = Integer.parseInt(OutTempAndSign.substring(0,1));
			            OutTemp = (SignOutTemp==1) ? ((-1)*(OutTempBC/10+OutTempA*10)) : (OutTempBC/10+OutTempA*10);
		            }

		            if (packetData.hasMoreTokens()){
		                RelHum = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                DewPoint = Integer.parseInt(packetData.nextToken());
		            }
		            
		            try {
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("outConditions",new Value(Data), new Value(OutTemp), new Value(RelHum), new Value(DewPoint)));
			            }
			            catch (Exception e){System.out.println("Errore parse 3: "+e);}
			            
        		}			
        break;
        case 4:		
        break;
        case 5:		
		            /*    
		            Example: 00 11 02 46 09 dc 0c 50 79 16
		            
		            Byte	Nibble	Bit	Meaning
		            05. 1	Bx		Battery status. Higher value == lower battery volt
		            05. 1   xB		Unknown
		            05. 2	DD		Inside temp, bc of -?<ab.c<? Celsius
		            05. 3	xD		Inside temp, a  of -?<ab.c<? Celsius
		            05. 3	Bx	3	Sign of temp, 1 = negative
		            05. 4	DD		Relative humidity, ab of ?<ab<? percent
		            05. 5	DD		Dew point, ab of 0<ab<? Celsius
		            05. 6	HH		Baro pressure, convert to decimal and add 795. mb.
		            05. 7	Bx		Unknown
		            05. 7	xB		Encoded 'tendency' 0x0c=clear 0x06=partly cloudy
		            0x02=cloudy 0x03=rain
		            05. 8	DD		Sea level reference, cd of <abc.d>. 
		            05. 9	DD		Sea level reference, ab of <abc.d>. Add this to raw
		            bp from byte 6 to get sea level pressure.
		            
		            Example: 21.1 Celsius, 46% humidity, dew point 9 Celsius
		            BP 1015 mb, sea level pressure 1015 mb.
		            */
            
		        {try {
		            if (packetData.hasMoreTokens()){
		                BatteryStatus = Integer.parseInt(packetData.nextToken().substring(0,1));
		            }
		            
		            if (packetData.hasMoreTokens()){
		                InTempBC = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                InTempAndSign = packetData.nextToken();
		                InTempA = Integer.parseInt(InTempAndSign.substring(1,2));
			            SignInTemp = Integer.parseInt(InTempAndSign.substring(0,1));
			            InTemp = (SignInTemp==1) ? ((-1)*(InTempBC/10+InTempA*10)) : (InTempBC/10+InTempA*10);
		            }
		            
		            if (packetData.hasMoreTokens()){
		                RelHum = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                DewPoint = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                BaroPress = Integer.parseInt(packetData.nextToken(),16);
		                BaroPress += 795;
		            }
		            if (packetData.hasMoreTokens()){
		                Tendency = Integer.parseInt(packetData.nextToken().substring(1,2),16);
		            }
		            if (packetData.hasMoreTokens()){
		                SeaLevelCD = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                SeaLevelAB = Integer.parseInt(packetData.nextToken());
		                SeaLevel = SeaLevelAB*10 + SeaLevelCD/10;
		            }
		            
		            
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("inConditions",new Value(Data), new Value(InTemp), new Value(RelHum), new Value(DewPoint), new Value(BaroPress), new Value(Tendency), new Value(SeaLevel)));
			            }
			            catch (Exception e){System.out.println("Errore parse 5: "+e);}
			                 
		        }
        break;		          
        case 6:	{
		            /*    
		            Type 06
		            
		            Example: 00 29 02 41 09 8b 61 90 33 06 2e
		            		  40 05 02 54 11 a7 21 00 00 06 7e
		            Byte	Nibble	Bit	Meaning
		            06. 1	Bx		Battery status. Higher value == lower battery volt
		            06. 1   xB		Unknown
		            06. 2	DD		Inside temp, bc of -?<ab.c<? Celsius
		            06. 3	xD		Inside temp, a  of -?<ab.c<? Celsius
		            06. 3	Bx	3	Sign of temp, 1 = negative
		            06. 4	DD		Relative humidity, ab of ?<ab<? percent
		            06. 5	DD		Dew point, ab of 0<ab<? Celsius
		            06. 6	HH		Baro pressure, low byte (bc of abc)
		            06. 7	xB	0 	Baro pressure, a of abc. Convert abc from hex
		   				         to decimal, add 600. to get mb. 
		   				         a is low bit only.
		   				         Upper three bits may be encoded BP tendency.
		            06. 7	Bx		Encoded 'tendency' 0x0c=clear 0x06=partly cloudy
		            				0x02=cloudy 0x03=rain
		            06. 8	DD		Sea level reference, ef of <abcd.ef>. 
		            06. 9	DD		Sea level reference, cd of <abcd.ef>. 
		            06.10   DD     Sea level reference, ab of <abcd.ef> Add this to raw
		            				bp from byte 6 and 7 to get sea level pressure.
		            
		            Example: 22.9 Celsius, 41% humidity, dew point 9 Celsius
		            BP 995 mb, sea level pressure 1028.9 mb.
		            */
            try {
		            if (packetData.hasMoreTokens()){
		                BatteryStatus = Integer.parseInt(packetData.nextToken().substring(0,1));
		            }
		            
		            if (packetData.hasMoreTokens()){
		                InTempBC = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                InTempAndSign = packetData.nextToken();
		                InTempA = Integer.parseInt(InTempAndSign.substring(1,2));
		                SignInTemp = Integer.parseInt(InTempAndSign.substring(0,1));
		                InTemp = (SignInTemp==1) ? ((-1)*(InTempBC/10+InTempA*10)) : (InTempBC/10+InTempA*10);
		            }
		            
		            if (packetData.hasMoreTokens()){
		                RelHum = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                DewPoint = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                BaroPressBC = Integer.parseInt(packetData.nextToken(),16);
		            }
		            if (packetData.hasMoreTokens()){
		                BaroPressAndTendency = packetData.nextToken();
		                BaroPressBinA = ""+Integer.parseInt(BaroPressAndTendency.substring(1,2));
		                Tendency = Integer.parseInt(BaroPressAndTendency.substring(0,1),16);
		                BaroPress = BaroPressBC + BaroPressA*100 + 600;
		            }
		            /*
		            if (packetData.hasMoreTokens()){
		                SeaLevelEF = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                SeaLevelCD = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                SeaLevelAB = Integer.parseInt(packetData.nextToken());
		                SeaLevel = SeaLevelAB*100 + SeaLevelCD + SeaLevelEF/100;
		            }*/
		            

		           
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("inConditions",new Value(Data), new Value(InTemp), new Value(RelHum), new Value(DewPoint), new Value(BaroPress), new Value(Tendency), new Value(SeaLevel)));
			            }
			            catch (Exception e){System.out.println("Errore parse 6: "+e);}
        		}	
        break;
        case 7:		 
        break;
        case 8:		
        case 9:		
        break;
        case 10:	 		
        break;		
        case 11:	 
        break;
        case 12:	
        break;
        case 13:	
        break;
        case 14:	
		            /*    
		            Type E
		            
		            Example: 81 8d
		            
		            Byte	Nibble	Bit	Meaning
		            0e. 1	Bx		Status, high bit, battery for main unit
		            0e. 1	DD		After removing high bit, minute chime
		            
		            Example: status = 1, time is hh:01
		            */
            
        		{  try {
		            if (packetData.hasMoreTokens()){
		                BatteryStatus = Integer.parseInt(packetData.nextToken().substring(0,1));
		            }
		            
		            if (packetData.hasMoreTokens()){
		                MinuteChime = Integer.parseInt(packetData.nextToken(),16);
		            }
		            
		          
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("battery",new Value(Data), new Value(BatteryStatus)));
			            }
			            catch (Exception e){System.out.println("Errore parse 14: "+e);}
        		}
            
        				
        break;		          
        case 15:	
		            /*    
		            Type F
		            
		            Example: 80 07 09 03 00 a0
		            
		            Byte	Nibble	Bit	Meaning
		            0f. 1	BB		Status. High bit == battery low
		            0f. 2	DD		Hour 
		            0f. 3	DD		Day 
		            0f. 4	DD		Month 
		            0f. 5	DD		Year 
		            
		            Example: It's 7AM, 9-March-2000
		            */
            
        		{ try {
		            if (packetData.hasMoreTokens()){
		                BatteryStatus = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                Hour = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                Day = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                Month = Integer.parseInt(packetData.nextToken());
		            }
		            if (packetData.hasMoreTokens()){
		                Year = Integer.parseInt(packetData.nextToken());
		            }
		            
		           
			            TucsonContext context = Tucson.enterDefaultContext();
			            context.out(tc, new LogicTuple("hourlyStatusReport",new Value(Hour), new Value("time",new Value(Day),new Value(Month),new Value(Year))));
			            }
			            catch (Exception e){System.out.println("Errore parse 15: "+e);}
        		
				}		
        break;
        default:	
        }

    }
}
